@@ -7,6 +7,7 @@
 #include <string.h>
 #include <stdio.h>
 #include <ctype.h>
+#include <limits.h>
 #include <assert.h>
 
 #include "server.h"
@@ -74,22 +75,25 @@ static int config_insert(server *srv) {
 		{ "debug.log-request-handling",  NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_SERVER },     /* 32 */
 		{ "debug.log-response-header",   NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_SERVER },     /* 33 */
 		{ "debug.log-request-header",    NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_SERVER },     /* 34 */
+		{ "debug.log-ssl-noise",         NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_SERVER },     /* 35 */
 
-		{ "server.protocol-http11",      NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_SERVER },     /* 35 */
-		{ "debug.log-request-header-on-error", NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_SERVER }, /* 36 */
-		{ "debug.log-state-handling",    NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_SERVER },     /* 37 */
-		{ "ssl.ca-file",                 NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_SERVER },      /* 38 */
-
-		{ "server.errorlog-use-syslog",  NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_SERVER },     /* 39 */
-		{ "server.range-requests",       NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION }, /* 40 */
-		{ "server.stat-cache-engine",    NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },  /* 41 */
-		{ "server.max-connections",      NULL, T_CONFIG_SHORT, T_CONFIG_SCOPE_SERVER },       /* 42 */
-		{ "server.network-backend",      NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },  /* 43 */
-		{ "server.upload-dirs",          NULL, T_CONFIG_ARRAY, T_CONFIG_SCOPE_CONNECTION },   /* 44 */
-		{ "server.core-files",           NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION }, /* 45 */
-		{ "ssl.cipher-list",             NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_SERVER },      /* 46 */
-		{ "ssl.use-sslv2",               NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION }, /* 47 */
+		{ "server.protocol-http11",      NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_SERVER },     /* 36 */
+		{ "debug.log-request-header-on-error", NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_SERVER }, /* 37 */
+		{ "debug.log-state-handling",    NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_SERVER },     /* 38 */
+		{ "ssl.ca-file",                 NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_SERVER },      /* 39 */
 
+		{ "server.errorlog-use-syslog",  NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_SERVER },     /* 40 */
+		{ "server.range-requests",       NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION }, /* 41 */
+		{ "server.stat-cache-engine",    NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },  /* 42 */
+		{ "server.max-connections",      NULL, T_CONFIG_SHORT, T_CONFIG_SCOPE_SERVER },       /* 43 */
+		{ "server.network-backend",      NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },  /* 44 */
+		{ "server.upload-dirs",          NULL, T_CONFIG_ARRAY, T_CONFIG_SCOPE_CONNECTION },   /* 45 */
+		{ "server.core-files",           NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION }, /* 46 */
+		{ "ssl.cipher-list",             NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_SERVER },      /* 47 */
+		{ "ssl.use-sslv2",               NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION }, /* 48 */
+		{ "etag.use-inode",             NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_SERVER }, /* 49 */
+		{ "etag.use-mtime",             NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_SERVER }, /* 50 */
+		{ "etag.use-size",             NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_SERVER }, /* 51 */
 		{ "server.host",                 "use server.bind instead", T_CONFIG_DEPRECATED, T_CONFIG_SCOPE_UNSET },
 		{ "server.docroot",              "use server.document-root instead", T_CONFIG_DEPRECATED, T_CONFIG_SCOPE_UNSET },
 		{ "server.virtual-root",         "load mod_simple_vhost and use simple-vhost.server-root instead", T_CONFIG_DEPRECATED, T_CONFIG_SCOPE_UNSET },
@@ -118,18 +122,18 @@ static int config_insert(server *srv) {
 
 	cv[13].destination = &(srv->srvconf.max_worker);
 	cv[23].destination = &(srv->srvconf.max_fds);
-	cv[36].destination = &(srv->srvconf.log_request_header_on_error);
-	cv[37].destination = &(srv->srvconf.log_state_handling);
+	cv[37].destination = &(srv->srvconf.log_request_header_on_error);
+	cv[38].destination = &(srv->srvconf.log_state_handling);
 
-	cv[39].destination = &(srv->srvconf.errorlog_use_syslog);
+	cv[40].destination = &(srv->srvconf.errorlog_use_syslog);
 
 	stat_cache_string = buffer_init();
-	cv[41].destination = stat_cache_string;
-	cv[43].destination = srv->srvconf.network_backend;
-	cv[44].destination = srv->srvconf.upload_tempdirs;
-	cv[45].destination = &(srv->srvconf.enable_cores);
+	cv[42].destination = stat_cache_string;
+	cv[44].destination = srv->srvconf.network_backend;
+	cv[45].destination = srv->srvconf.upload_tempdirs;
+	cv[46].destination = &(srv->srvconf.enable_cores);
 
-	cv[42].destination = &(srv->srvconf.max_conns);
+	cv[43].destination = &(srv->srvconf.max_conns);
 	cv[12].destination = &(srv->srvconf.max_request_size);
 	srv->config_storage = calloc(1, srv->config_context->used * sizeof(specific_config *));
 
@@ -162,6 +166,9 @@ static int config_insert(server *srv) {
 #endif
 		s->kbytes_per_second = 0;
 		s->allow_http11  = 1;
+		s->etag_use_inode = 1;
+		s->etag_use_mtime = 1;
+		s->etag_use_size  = 1;
 		s->range_requests = 1;
 		s->force_lowercase_filenames = 0;
 		s->global_kbytes_per_second = 0;
@@ -199,13 +206,17 @@ static int config_insert(server *srv) {
 		cv[32].destination = &(s->log_request_handling);
 		cv[33].destination = &(s->log_response_header);
 		cv[34].destination = &(s->log_request_header);
+		cv[35].destination = &(s->log_ssl_noise);
 
-		cv[35].destination = &(s->allow_http11);
-		cv[38].destination = s->ssl_ca_file;
-		cv[40].destination = &(s->range_requests);
+		cv[36].destination = &(s->allow_http11);
+		cv[39].destination = s->ssl_ca_file;
+		cv[41].destination = &(s->range_requests);
 
-		cv[46].destination = s->ssl_cipher_list;
-		cv[47].destination = &(s->ssl_use_sslv2);
+		cv[47].destination = s->ssl_cipher_list;
+		cv[48].destination = &(s->ssl_use_sslv2);
+		cv[49].destination = &(s->etag_use_inode);
+		cv[50].destination = &(s->etag_use_mtime);
+		cv[51].destination = &(s->etag_use_size);
 
 		srv->config_storage[i] = s;
 
@@ -271,6 +282,7 @@ int config_setup_connection(server *srv, connection *con) {
 	PATCH(log_request_handling);
 	PATCH(log_condition_handling);
 	PATCH(log_file_not_found);
+	PATCH(log_ssl_noise);
 
 	PATCH(range_requests);
 	PATCH(force_lowercase_filenames);
@@ -280,22 +292,23 @@ int config_setup_connection(server *srv, connection *con) {
 	PATCH(ssl_ca_file);
 	PATCH(ssl_cipher_list);
 	PATCH(ssl_use_sslv2);
-
-
+	PATCH(etag_use_inode);
+	PATCH(etag_use_mtime);
+	PATCH(etag_use_size);
+ 
 	return 0;
 }
 
 int config_patch_connection(server *srv, connection *con, comp_key_t comp) {
 	size_t i, j;
 
+	con->conditional_is_valid[comp] = 1;
+
 	/* skip the first, the global context */
 	for (i = 1; i < srv->config_context->used; i++) {
 		data_config *dc = (data_config *)srv->config_context->data[i];
 		specific_config *s = srv->config_storage[i];
 
-		/* not our stage */
-		if (comp != dc->comp) continue;
-
 		/* condition didn't match */
 		if (!config_check_cond(srv, con, dc)) continue;
 
@@ -323,6 +336,12 @@ int config_patch_connection(server *srv, connection *con, comp_key_t comp) {
 				PATCH(max_read_idle);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("mimetype.use-xattr"))) {
 				PATCH(use_xattr);
+			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("etag.use-inode"))) {
+				PATCH(etag_use_inode);
+			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("etag.use-mtime"))) {
+				PATCH(etag_use_mtime);
+			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("etag.use-size"))) {
+				PATCH(etag_use_size);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("ssl.pemfile"))) {
 				PATCH(ssl_pemfile);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("ssl.ca-file"))) {
@@ -353,6 +372,8 @@ int config_patch_connection(server *srv, connection *con, comp_key_t comp) {
 				PATCH(log_condition_handling);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("debug.log-file-not-found"))) {
 				PATCH(log_file_not_found);
+			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("debug.log-ssl-noise"))) {
+				PATCH(log_ssl_noise);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("server.protocol-http11"))) {
 				PATCH(allow_http11);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("server.force-lowercase-filenames"))) {
@@ -365,6 +386,10 @@ int config_patch_connection(server *srv, connection *con, comp_key_t comp) {
 		}
 	}
 
+		con->etag_flags = (con->conf.etag_use_mtime ? ETAG_USE_MTIME : 0) |
+				  (con->conf.etag_use_inode ? ETAG_USE_INODE : 0) |
+				  (con->conf.etag_use_size  ? ETAG_USE_SIZE  : 0);
+
 	return 0;
 }
 #undef PATCH
@@ -388,8 +413,8 @@ typedef struct {
 
 #if 0
 static int tokenizer_open(server *srv, tokenizer_t *t, buffer *basedir, const char *fn) {
-	if (buffer_is_empty(basedir) &&
-			(fn[0] == '/' || fn[0] == '\\') &&
+	if (buffer_is_empty(basedir) ||
+			(fn[0] == '/' || fn[0] == '\\') ||
 			(fn[0] == '.' && (fn[1] == '/' || fn[1] == '\\'))) {
 		t->file = buffer_init_string(fn);
 	} else {
@@ -458,7 +483,7 @@ static int config_tokenizer(server *srv, tokenizer_t *t, int *token_id, buffer *
 				if (t->input[t->offset + 1] == '>') {
 					t->offset += 2;
 
-					buffer_copy_string(token, "=>");
+					buffer_copy_string_len(token, CONST_STR_LEN("=>"));
 
 					tid = TK_ARRAY_ASSIGN;
 				} else {
@@ -472,13 +497,13 @@ static int config_tokenizer(server *srv, tokenizer_t *t, int *token_id, buffer *
 				if (t->input[t->offset + 1] == '=') {
 					t->offset += 2;
 
-					buffer_copy_string(token, "==");
+					buffer_copy_string_len(token, CONST_STR_LEN("=="));
 
 					tid = TK_EQ;
 				} else if (t->input[t->offset + 1] == '~') {
 					t->offset += 2;
 
-					buffer_copy_string(token, "=~");
+					buffer_copy_string_len(token, CONST_STR_LEN("=~"));
 
 					tid = TK_MATCH;
 				} else {
@@ -511,13 +536,13 @@ static int config_tokenizer(server *srv, tokenizer_t *t, int *token_id, buffer *
 				if (t->input[t->offset + 1] == '=') {
 					t->offset += 2;
 
-					buffer_copy_string(token, "!=");
+					buffer_copy_string_len(token, CONST_STR_LEN("!="));
 
 					tid = TK_NE;
 				} else if (t->input[t->offset + 1] == '~') {
 					t->offset += 2;
 
-					buffer_copy_string(token, "!~");
+					buffer_copy_string_len(token, CONST_STR_LEN("!~"));
 
 					tid = TK_NOMATCH;
 				} else {
@@ -572,7 +597,7 @@ static int config_tokenizer(server *srv, tokenizer_t *t, int *token_id, buffer *
 				}
 				t->in_key = 1;
 				tid = TK_EOL;
-				buffer_copy_string(token, "(EOL)");
+				buffer_copy_string_len(token, CONST_STR_LEN("(EOL)"));
 			} else {
 				config_skip_newline(t);
 				t->line_pos = 1;
@@ -583,7 +608,7 @@ static int config_tokenizer(server *srv, tokenizer_t *t, int *token_id, buffer *
 			if (t->in_brace > 0) {
 				tid = TK_COMMA;
 
-				buffer_copy_string(token, "(COMMA)");
+				buffer_copy_string_len(token, CONST_STR_LEN("(COMMA)"));
 			}
 
 			t->offset++;
@@ -592,7 +617,7 @@ static int config_tokenizer(server *srv, tokenizer_t *t, int *token_id, buffer *
 		case '"':
 			/* search for the terminating " */
 			start = t->input + t->offset + 1;
-			buffer_copy_string(token, "");
+			buffer_copy_string_len(token, CONST_STR_LEN(""));
 
 			for (i = 1; t->input[t->offset + i]; i++) {
 				if (t->input[t->offset + i] == '\\' &&
@@ -638,7 +663,7 @@ static int config_tokenizer(server *srv, tokenizer_t *t, int *token_id, buffer *
 
 			tid = TK_LPARAN;
 
-			buffer_copy_string(token, "(");
+			buffer_copy_string_len(token, CONST_STR_LEN("("));
 			break;
 		case ')':
 			t->offset++;
@@ -646,7 +671,7 @@ static int config_tokenizer(server *srv, tokenizer_t *t, int *token_id, buffer *
 
 			tid = TK_RPARAN;
 
-			buffer_copy_string(token, ")");
+			buffer_copy_string_len(token, CONST_STR_LEN(")"));
 			break;
 		case '$':
 			t->offset++;
@@ -655,19 +680,19 @@ static int config_tokenizer(server *srv, tokenizer_t *t, int *token_id, buffer *
 			t->in_cond = 1;
 			t->in_key = 0;
 
-			buffer_copy_string(token, "$");
+			buffer_copy_string_len(token, CONST_STR_LEN("$"));
 
 			break;
 
 		case '+':
 			if (t->input[t->offset + 1] == '=') {
 				t->offset += 2;
-				buffer_copy_string(token, "+=");
+				buffer_copy_string_len(token, CONST_STR_LEN("+="));
 				tid = TK_APPEND;
 			} else {
 				t->offset++;
 				tid = TK_PLUS;
-				buffer_copy_string(token, "+");
+				buffer_copy_string_len(token, CONST_STR_LEN("+"));
 			}
 			break;
 
@@ -676,7 +701,7 @@ static int config_tokenizer(server *srv, tokenizer_t *t, int *token_id, buffer *
 
 			tid = TK_LCURLY;
 
-			buffer_copy_string(token, "{");
+			buffer_copy_string_len(token, CONST_STR_LEN("{"));
 
 			break;
 
@@ -685,7 +710,7 @@ static int config_tokenizer(server *srv, tokenizer_t *t, int *token_id, buffer *
 
 			tid = TK_RCURLY;
 
-			buffer_copy_string(token, "}");
+			buffer_copy_string_len(token, CONST_STR_LEN("}"));
 
 			break;
 
@@ -694,7 +719,7 @@ static int config_tokenizer(server *srv, tokenizer_t *t, int *token_id, buffer *
 
 			tid = TK_LBRACKET;
 
-			buffer_copy_string(token, "[");
+			buffer_copy_string_len(token, CONST_STR_LEN("["));
 
 			break;
 
@@ -703,7 +728,7 @@ static int config_tokenizer(server *srv, tokenizer_t *t, int *token_id, buffer *
 
 			tid = TK_RBRACKET;
 
-			buffer_copy_string(token, "]");
+			buffer_copy_string_len(token, CONST_STR_LEN("]"));
 
 			break;
 		case '#':
@@ -862,8 +887,8 @@ int config_parse_file(server *srv, config_t *context, const char *fn) {
 	int ret;
 	buffer *filename;
 
-	if (buffer_is_empty(context->basedir) &&
-			(fn[0] == '/' || fn[0] == '\\') &&
+	if (buffer_is_empty(context->basedir) ||
+			(fn[0] == '/' || fn[0] == '\\') ||
 			(fn[0] == '.' && (fn[1] == '/' || fn[1] == '\\'))) {
 		filename = buffer_init_string(fn);
 	} else {
@@ -890,15 +915,39 @@ int config_parse_file(server *srv, config_t *context, const char *fn) {
 	return ret;
 }
 
+static char* getCWD() {
+	char *s, *s1;
+	size_t len;
+#ifdef PATH_MAX
+	len = PATH_MAX;
+#else
+	len = 4096;
+#endif
+
+	s = malloc(len);
+	if (!s) return NULL;
+	while (NULL == getcwd(s, len)) {
+		if (errno != ERANGE || SSIZE_MAX - len < len) return NULL;
+		len *= 2;
+		s1 = realloc(s, len);
+		if (!s1) {
+			free(s);
+			return NULL;
+		}
+		s = s1;
+	}
+	return s;
+}
+
 int config_parse_cmd(server *srv, config_t *context, const char *cmd) {
 	proc_handler_t proc;
 	tokenizer_t t;
 	int ret;
 	buffer *source;
 	buffer *out;
-	char oldpwd[PATH_MAX];
+	char *oldpwd;
 
-	if (NULL == getcwd(oldpwd, sizeof(oldpwd))) {
+	if (NULL == (oldpwd = getCWD())) {
 		log_error_write(srv, __FILE__, __LINE__, "s",
 				"cannot get cwd", strerror(errno));
 		return -1;
@@ -923,6 +972,7 @@ int config_parse_cmd(server *srv, config_t *context, const char *cmd) {
 	buffer_free(source);
 	buffer_free(out);
 	chdir(oldpwd);
+	free(oldpwd);
 	return ret;
 }
 
@@ -964,7 +1014,7 @@ int config_read(server *srv, const char *fn) {
 	}
 
 	dc = data_config_init();
-	buffer_copy_string(dc->key, "global");
+	buffer_copy_string_len(dc->key, CONST_STR_LEN("global"));
 
 	assert(context.all_configs->used == 0);
 	dc->context_ndx = context.all_configs->used;
@@ -975,14 +1025,14 @@ int config_read(server *srv, const char *fn) {
 	srv->config = dc->value;
 	dpid = data_integer_init();
 	dpid->value = getpid();
-	buffer_copy_string(dpid->key, "var.PID");
+	buffer_copy_string_len(dpid->key, CONST_STR_LEN("var.PID"));
 	array_insert_unique(srv->config, (data_unset *)dpid);
 
 	dcwd = data_string_init();
 	buffer_prepare_copy(dcwd->value, 1024);
 	if (NULL != getcwd(dcwd->value->ptr, dcwd->value->size - 1)) {
 		dcwd->value->used = strlen(dcwd->value->ptr) + 1;
-		buffer_copy_string(dcwd->key, "var.CWD");
+		buffer_copy_string_len(dcwd->key, CONST_STR_LEN("var.CWD"));
 		array_insert_unique(srv->config, (data_unset *)dcwd);
 	}
 
@@ -1016,7 +1066,7 @@ int config_read(server *srv, const char *fn) {
 		/* prepend default modules */
 		if (NULL == array_get_element(modules->value, "mod_indexfile")) {
 			ds = data_string_init();
-			buffer_copy_string(ds->value, "mod_indexfile");
+			buffer_copy_string_len(ds->value, CONST_STR_LEN("mod_indexfile"));
 			array_insert_unique(prepends->value, (data_unset *)ds);
 		}
 
@@ -1029,13 +1079,13 @@ int config_read(server *srv, const char *fn) {
 		/* append default modules */
 		if (NULL == array_get_element(modules->value, "mod_dirlisting")) {
 			ds = data_string_init();
-			buffer_copy_string(ds->value, "mod_dirlisting");
+			buffer_copy_string_len(ds->value, CONST_STR_LEN("mod_dirlisting"));
 			array_insert_unique(modules->value, (data_unset *)ds);
 		}
 
 		if (NULL == array_get_element(modules->value, "mod_staticfile")) {
 			ds = data_string_init();
-			buffer_copy_string(ds->value, "mod_staticfile");
+			buffer_copy_string_len(ds->value, CONST_STR_LEN("mod_staticfile"));
 			array_insert_unique(modules->value, (data_unset *)ds);
 		}
 	} else {
@@ -1045,18 +1095,18 @@ int config_read(server *srv, const char *fn) {
 
 		/* server.modules is not set */
 		ds = data_string_init();
-		buffer_copy_string(ds->value, "mod_indexfile");
+		buffer_copy_string_len(ds->value, CONST_STR_LEN("mod_indexfile"));
 		array_insert_unique(modules->value, (data_unset *)ds);
 
 		ds = data_string_init();
-		buffer_copy_string(ds->value, "mod_dirlisting");
+		buffer_copy_string_len(ds->value, CONST_STR_LEN("mod_dirlisting"));
 		array_insert_unique(modules->value, (data_unset *)ds);
 
 		ds = data_string_init();
-		buffer_copy_string(ds->value, "mod_staticfile");
+		buffer_copy_string_len(ds->value, CONST_STR_LEN("mod_staticfile"));
 		array_insert_unique(modules->value, (data_unset *)ds);
 
-		buffer_copy_string(modules->key, "server.modules");
+		buffer_copy_string_len(modules->key, CONST_STR_LEN("server.modules"));
 		array_insert_unique(srv->config, (data_unset *)modules);
 	}
 
