@@ -24,6 +24,10 @@ static void configparser_push(config_t *ctx, data_config *dc, int isnew) {
     dc->parent = ctx->current;
     array_insert_unique(dc->parent->childs, (data_unset *)dc);
   }
+  if (ctx->configs_stack->used > 0 && ctx->current->context_ndx == 0) {
+    fprintf(stderr, "Cannot use conditionals inside a global { ... } block\n");
+    exit(-1);
+  }
   array_insert_unique(ctx->configs_stack, (data_unset *)ctx->current);
   ctx->current = dc;
 }
@@ -54,7 +58,7 @@ static data_unset *configparser_get_variable(config_t *ctx, const buffer *key) {
   return NULL;
 }
 
-/* op1 is to be eat/return by this function, op1->key is not cared
+/* op1 is to be eat/return by this function if success, op1->key is not cared
    op2 is left untouch, unreferenced
  */
 data_unset *configparser_merge_data(data_unset *op1, const data_unset *op2) {
@@ -72,7 +76,6 @@ data_unset *configparser_merge_data(data_unset *op1, const data_unset *op2) {
       return (data_unset *)ds;
     } else {
       fprintf(stderr, "data type mismatch, cannot be merge\n");
-      op1->free(op1);
       return NULL;
     }
   }
@@ -106,7 +109,7 @@ data_unset *configparser_merge_data(data_unset *op1, const data_unset *op2) {
 }
 
 
-#line 110 "configparser.c"
+#line 113 "configparser.c"
 /* Next is all token values, in a form suitable for use by makeheaders.
 ** This section will be null unless lemon is run with the -m switch.
 */
@@ -440,7 +443,7 @@ static const char *yyRuleName[] = {
 */
 const char *configparserTokenName(int tokenType){
 #ifndef NDEBUG
-  if( tokenType>0 && tokenType<(sizeof(yyTokenName)/sizeof(yyTokenName[0])) ){
+  if( tokenType>0 && (size_t)tokenType<(sizeof(yyTokenName)/sizeof(yyTokenName[0])) ){
     return yyTokenName[tokenType];
   }else{
     return "Unknown";
@@ -513,44 +516,44 @@ static void yy_destructor(YYCODETYPE yymajor, YYMINORTYPE *yypminor){
     case 23:
     case 24:
     case 25:
-#line 142 "./configparser.y"
+#line 146 "./configparser.y"
 { buffer_free((yypminor->yy0)); }
-#line 518 "configparser.c"
+#line 521 "configparser.c"
       break;
     case 35:
-#line 133 "./configparser.y"
+#line 137 "./configparser.y"
 { (yypminor->yy41)->free((yypminor->yy41)); }
-#line 523 "configparser.c"
+#line 526 "configparser.c"
       break;
     case 36:
-#line 134 "./configparser.y"
+#line 138 "./configparser.y"
 { (yypminor->yy41)->free((yypminor->yy41)); }
-#line 528 "configparser.c"
+#line 531 "configparser.c"
       break;
     case 37:
-#line 135 "./configparser.y"
+#line 139 "./configparser.y"
 { (yypminor->yy41)->free((yypminor->yy41)); }
-#line 533 "configparser.c"
+#line 536 "configparser.c"
       break;
     case 39:
-#line 136 "./configparser.y"
+#line 140 "./configparser.y"
 { array_free((yypminor->yy40)); }
-#line 538 "configparser.c"
+#line 541 "configparser.c"
       break;
     case 40:
-#line 137 "./configparser.y"
+#line 141 "./configparser.y"
 { array_free((yypminor->yy40)); }
-#line 543 "configparser.c"
+#line 546 "configparser.c"
       break;
     case 41:
-#line 138 "./configparser.y"
+#line 142 "./configparser.y"
 { buffer_free((yypminor->yy43)); }
-#line 548 "configparser.c"
+#line 551 "configparser.c"
       break;
     case 42:
-#line 139 "./configparser.y"
+#line 143 "./configparser.y"
 { buffer_free((yypminor->yy43)); }
-#line 553 "configparser.c"
+#line 556 "configparser.c"
       break;
     default:  break;   /* If no destructor action specified: do nothing */
   }
@@ -628,7 +631,7 @@ static int yy_find_shift_action(
     return YY_NO_ACTION;
   }
   i += iLookAhead;
-  if( i<0 || i>=YY_SZ_ACTTAB || yy_lookahead[i]!=iLookAhead ){
+  if( i<0 || (size_t)i>=YY_SZ_ACTTAB || yy_lookahead[i]!=iLookAhead ){
 #ifdef YYFALLBACK
     int iFallback;            /* Fallback token */
     if( iLookAhead<sizeof(yyFallback)/sizeof(yyFallback[0])
@@ -671,7 +674,7 @@ static int yy_find_reduce_action(
     return YY_NO_ACTION;
   }
   i += iLookAhead;
-  if( i<0 || i>=YY_SZ_ACTTAB || yy_lookahead[i]!=iLookAhead ){
+  if( i<0 || (size_t)i>=YY_SZ_ACTTAB || yy_lookahead[i]!=iLookAhead ){
     return yy_default[stateno];
   }else{
     return yy_action[i];
@@ -787,7 +790,7 @@ static void yy_reduce(
   yymsp = &yypParser->yystack[yypParser->yyidx];
 #ifndef NDEBUG
   if( yyTraceFILE && yyruleno>=0
-        && yyruleno<sizeof(yyRuleName)/sizeof(yyRuleName[0]) ){
+        && (size_t)yyruleno<sizeof(yyRuleName)/sizeof(yyRuleName[0]) ){
     fprintf(yyTraceFILE, "%sReduce [%s].\n", yyTracePrompt,
       yyRuleName[yyruleno]);
   }
@@ -818,9 +821,9 @@ static void yy_reduce(
         /* No destructor defined for global */
         break;
       case 5:
-#line 116 "./configparser.y"
+#line 119 "./configparser.y"
 { yymsp[-1].minor.yy78 = NULL; }
-#line 823 "configparser.c"
+#line 826 "configparser.c"
   yy_destructor(1,&yymsp[0].minor);
         break;
       case 6:
@@ -833,33 +836,35 @@ static void yy_reduce(
   yy_destructor(1,&yymsp[0].minor);
         break;
       case 9:
-#line 144 "./configparser.y"
+#line 148 "./configparser.y"
 {
-  buffer_copy_string_buffer(yymsp[0].minor.yy41->key, yymsp[-2].minor.yy43);
-  if (strncmp(yymsp[-2].minor.yy43->ptr, "env.", sizeof("env.") - 1) == 0) {
-    fprintf(stderr, "Setting env variable is not supported in conditional %d %s: %s\n",
-        ctx->current->context_ndx,
-        ctx->current->key->ptr, yymsp[-2].minor.yy43->ptr);
-    ctx->ok = 0;
-  } else if (NULL == array_get_element(ctx->current->value, yymsp[0].minor.yy41->key->ptr)) {
-    array_insert_unique(ctx->current->value, yymsp[0].minor.yy41);
-    yymsp[0].minor.yy41 = NULL;
-  } else {
-    fprintf(stderr, "Duplicate config variable in conditional %d %s: %s\n",
-            ctx->current->context_ndx,
-            ctx->current->key->ptr, yymsp[0].minor.yy41->key->ptr);
-    ctx->ok = 0;
-    yymsp[0].minor.yy41->free(yymsp[0].minor.yy41);
-    yymsp[0].minor.yy41 = NULL;
+  if (ctx->ok) {
+    buffer_copy_string_buffer(yymsp[0].minor.yy41->key, yymsp[-2].minor.yy43);
+    if (strncmp(yymsp[-2].minor.yy43->ptr, "env.", sizeof("env.") - 1) == 0) {
+      fprintf(stderr, "Setting env variable is not supported in conditional %d %s: %s\n",
+          ctx->current->context_ndx,
+          ctx->current->key->ptr, yymsp[-2].minor.yy43->ptr);
+      ctx->ok = 0;
+    } else if (NULL == array_get_element(ctx->current->value, yymsp[0].minor.yy41->key->ptr)) {
+      array_insert_unique(ctx->current->value, yymsp[0].minor.yy41);
+      yymsp[0].minor.yy41 = NULL;
+    } else {
+      fprintf(stderr, "Duplicate config variable in conditional %d %s: %s\n",
+              ctx->current->context_ndx,
+              ctx->current->key->ptr, yymsp[0].minor.yy41->key->ptr);
+      ctx->ok = 0;
+      yymsp[0].minor.yy41->free(yymsp[0].minor.yy41);
+      yymsp[0].minor.yy41 = NULL;
+    }
   }
   buffer_free(yymsp[-2].minor.yy43);
   yymsp[-2].minor.yy43 = NULL;
 }
-#line 858 "configparser.c"
+#line 863 "configparser.c"
   yy_destructor(2,&yymsp[-1].minor);
         break;
       case 10:
-#line 166 "./configparser.y"
+#line 172 "./configparser.y"
 {
   array *vars = ctx->current->value;
   data_unset *du;
@@ -884,6 +889,7 @@ static void yy_reduce(
     du = configparser_merge_data(du, yymsp[0].minor.yy41);
     if (NULL == du) {
       ctx->ok = 0;
+      du->free(du);
     }
     else {
       buffer_copy_string_buffer(du->key, yymsp[-2].minor.yy43);
@@ -898,11 +904,11 @@ static void yy_reduce(
   yymsp[-2].minor.yy43 = NULL;
   yymsp[0].minor.yy41 = NULL;
 }
-#line 901 "configparser.c"
+#line 907 "configparser.c"
   yy_destructor(3,&yymsp[-1].minor);
         break;
       case 11:
-#line 205 "./configparser.y"
+#line 212 "./configparser.y"
 {
   if (strchr(yymsp[0].minor.yy0->ptr, '.') == NULL) {
     yygotominor.yy43 = buffer_init_string("var.");
@@ -914,10 +920,10 @@ static void yy_reduce(
     yymsp[0].minor.yy0 = NULL;
   }
 }
-#line 917 "configparser.c"
+#line 923 "configparser.c"
         break;
       case 12:
-#line 217 "./configparser.y"
+#line 224 "./configparser.y"
 {
   yygotominor.yy41 = configparser_merge_data(yymsp[-2].minor.yy41, yymsp[0].minor.yy41);
   if (NULL == yygotominor.yy41) {
@@ -927,19 +933,19 @@ static void yy_reduce(
   yymsp[0].minor.yy41->free(yymsp[0].minor.yy41);
   yymsp[0].minor.yy41 = NULL;
 }
-#line 930 "configparser.c"
+#line 936 "configparser.c"
   yy_destructor(5,&yymsp[-1].minor);
         break;
       case 13:
-#line 227 "./configparser.y"
+#line 234 "./configparser.y"
 {
   yygotominor.yy41 = yymsp[0].minor.yy41;
   yymsp[0].minor.yy41 = NULL;
 }
-#line 939 "configparser.c"
+#line 945 "configparser.c"
         break;
       case 14:
-#line 232 "./configparser.y"
+#line 239 "./configparser.y"
 {
   yygotominor.yy41 = NULL;
   if (strncmp(yymsp[0].minor.yy43->ptr, "env.", sizeof("env.") - 1) == 0) {
@@ -966,59 +972,59 @@ static void yy_reduce(
   buffer_free(yymsp[0].minor.yy43);
   yymsp[0].minor.yy43 = NULL;
 }
-#line 969 "configparser.c"
+#line 975 "configparser.c"
         break;
       case 15:
-#line 259 "./configparser.y"
+#line 266 "./configparser.y"
 {
   yygotominor.yy41 = (data_unset *)data_string_init();
   buffer_copy_string_buffer(((data_string *)(yygotominor.yy41))->value, yymsp[0].minor.yy0);
   buffer_free(yymsp[0].minor.yy0);
   yymsp[0].minor.yy0 = NULL;
 }
-#line 979 "configparser.c"
+#line 985 "configparser.c"
         break;
       case 16:
-#line 266 "./configparser.y"
+#line 273 "./configparser.y"
 {
   yygotominor.yy41 = (data_unset *)data_integer_init();
   ((data_integer *)(yygotominor.yy41))->value = strtol(yymsp[0].minor.yy0->ptr, NULL, 10);
   buffer_free(yymsp[0].minor.yy0);
   yymsp[0].minor.yy0 = NULL;
 }
-#line 989 "configparser.c"
+#line 995 "configparser.c"
         break;
       case 17:
-#line 272 "./configparser.y"
+#line 279 "./configparser.y"
 {
   yygotominor.yy41 = (data_unset *)data_array_init();
   array_free(((data_array *)(yygotominor.yy41))->value);
   ((data_array *)(yygotominor.yy41))->value = yymsp[0].minor.yy40;
   yymsp[0].minor.yy40 = NULL;
 }
-#line 999 "configparser.c"
+#line 1005 "configparser.c"
         break;
       case 18:
-#line 278 "./configparser.y"
+#line 285 "./configparser.y"
 {
   yygotominor.yy40 = array_init();
 }
-#line 1006 "configparser.c"
+#line 1012 "configparser.c"
   yy_destructor(8,&yymsp[-1].minor);
   yy_destructor(9,&yymsp[0].minor);
         break;
       case 19:
-#line 281 "./configparser.y"
+#line 288 "./configparser.y"
 {
   yygotominor.yy40 = yymsp[-1].minor.yy40;
   yymsp[-1].minor.yy40 = NULL;
 }
-#line 1016 "configparser.c"
+#line 1022 "configparser.c"
   yy_destructor(8,&yymsp[-2].minor);
   yy_destructor(9,&yymsp[0].minor);
         break;
       case 20:
-#line 286 "./configparser.y"
+#line 293 "./configparser.y"
 {
   if (buffer_is_empty(yymsp[0].minor.yy41->key) ||
       NULL == array_get_element(yymsp[-2].minor.yy40, yymsp[0].minor.yy41->key->ptr)) {
@@ -1035,37 +1041,37 @@ static void yy_reduce(
   yygotominor.yy40 = yymsp[-2].minor.yy40;
   yymsp[-2].minor.yy40 = NULL;
 }
-#line 1038 "configparser.c"
+#line 1044 "configparser.c"
   yy_destructor(10,&yymsp[-1].minor);
         break;
       case 21:
-#line 303 "./configparser.y"
+#line 310 "./configparser.y"
 {
   yygotominor.yy40 = yymsp[-1].minor.yy40;
   yymsp[-1].minor.yy40 = NULL;
 }
-#line 1047 "configparser.c"
+#line 1053 "configparser.c"
   yy_destructor(10,&yymsp[0].minor);
         break;
       case 22:
-#line 308 "./configparser.y"
+#line 315 "./configparser.y"
 {
   yygotominor.yy40 = array_init();
   array_insert_unique(yygotominor.yy40, yymsp[0].minor.yy41);
   yymsp[0].minor.yy41 = NULL;
 }
-#line 1057 "configparser.c"
+#line 1063 "configparser.c"
         break;
       case 23:
-#line 314 "./configparser.y"
+#line 321 "./configparser.y"
 {
   yygotominor.yy41 = yymsp[0].minor.yy41;
   yymsp[0].minor.yy41 = NULL;
 }
-#line 1065 "configparser.c"
+#line 1071 "configparser.c"
         break;
       case 24:
-#line 318 "./configparser.y"
+#line 325 "./configparser.y"
 {
   buffer_copy_string_buffer(yymsp[0].minor.yy41->key, yymsp[-2].minor.yy43);
   buffer_free(yymsp[-2].minor.yy43);
@@ -1074,7 +1080,7 @@ static void yy_reduce(
   yygotominor.yy41 = yymsp[0].minor.yy41;
   yymsp[0].minor.yy41 = NULL;
 }
-#line 1077 "configparser.c"
+#line 1083 "configparser.c"
   yy_destructor(11,&yymsp[-1].minor);
         break;
       case 25:
@@ -1083,58 +1089,18 @@ static void yy_reduce(
       case 26:
         break;
       case 27:
-#line 330 "./configparser.y"
+#line 337 "./configparser.y"
 {
   data_config *dc;
   dc = (data_config *)array_get_element(ctx->srv->config_context, "global");
   assert(dc);
   configparser_push(ctx, dc, 0);
 }
-#line 1093 "configparser.c"
+#line 1099 "configparser.c"
   yy_destructor(12,&yymsp[0].minor);
         break;
       case 28:
-#line 337 "./configparser.y"
-{
-  data_config *cur;
-
-  cur = ctx->current;
-  configparser_pop(ctx);
-
-  assert(cur && ctx->current);
-
-  yygotominor.yy0 = cur;
-}
-#line 1108 "configparser.c"
-        /* No destructor defined for globalstart */
-  yy_destructor(13,&yymsp[-2].minor);
-        /* No destructor defined for metalines */
-  yy_destructor(14,&yymsp[0].minor);
-        break;
-      case 29:
-#line 348 "./configparser.y"
-{
-  assert(yymsp[-3].minor.yy78->context_ndx < yymsp[0].minor.yy78->context_ndx);
-  yymsp[0].minor.yy78->prev = yymsp[-3].minor.yy78;
-  yymsp[-3].minor.yy78->next = yymsp[0].minor.yy78;
-  yygotominor.yy78 = yymsp[0].minor.yy78;
-  yymsp[-3].minor.yy78 = NULL;
-  yymsp[0].minor.yy78 = NULL;
-}
-#line 1124 "configparser.c"
-        /* No destructor defined for eols */
-  yy_destructor(15,&yymsp[-1].minor);
-        break;
-      case 30:
-#line 357 "./configparser.y"
-{
-  yygotominor.yy78 = yymsp[0].minor.yy78;
-  yymsp[0].minor.yy78 = NULL;
-}
-#line 1134 "configparser.c"
-        break;
-      case 31:
-#line 362 "./configparser.y"
+#line 344 "./configparser.y"
 {
   data_config *cur;
 
@@ -1145,14 +1111,54 @@ static void yy_reduce(
 
   yygotominor.yy78 = cur;
 }
-#line 1148 "configparser.c"
+#line 1114 "configparser.c"
+        /* No destructor defined for globalstart */
+  yy_destructor(13,&yymsp[-2].minor);
+        /* No destructor defined for metalines */
+  yy_destructor(14,&yymsp[0].minor);
+        break;
+      case 29:
+#line 355 "./configparser.y"
+{
+  assert(yymsp[-3].minor.yy78->context_ndx < yymsp[0].minor.yy78->context_ndx);
+  yymsp[0].minor.yy78->prev = yymsp[-3].minor.yy78;
+  yymsp[-3].minor.yy78->next = yymsp[0].minor.yy78;
+  yygotominor.yy78 = yymsp[0].minor.yy78;
+  yymsp[-3].minor.yy78 = NULL;
+  yymsp[0].minor.yy78 = NULL;
+}
+#line 1130 "configparser.c"
+        /* No destructor defined for eols */
+  yy_destructor(15,&yymsp[-1].minor);
+        break;
+      case 30:
+#line 364 "./configparser.y"
+{
+  yygotominor.yy78 = yymsp[0].minor.yy78;
+  yymsp[0].minor.yy78 = NULL;
+}
+#line 1140 "configparser.c"
+        break;
+      case 31:
+#line 369 "./configparser.y"
+{
+  data_config *cur;
+
+  cur = ctx->current;
+  configparser_pop(ctx);
+
+  assert(cur && ctx->current);
+
+  yygotominor.yy78 = cur;
+}
+#line 1154 "configparser.c"
         /* No destructor defined for context */
   yy_destructor(13,&yymsp[-2].minor);
         /* No destructor defined for metalines */
   yy_destructor(14,&yymsp[0].minor);
         break;
       case 32:
-#line 373 "./configparser.y"
+#line 380 "./configparser.y"
 {
   data_config *dc;
   buffer *b, *rvalue, *op;
@@ -1201,10 +1207,15 @@ static void yy_reduce(
       { COMP_HTTP_URL,           CONST_STR_LEN("HTTP[\"url\"]"        ) },
       { COMP_HTTP_HOST,          CONST_STR_LEN("HTTP[\"host\"]"       ) },
       { COMP_HTTP_REFERER,       CONST_STR_LEN("HTTP[\"referer\"]"    ) },
-      { COMP_HTTP_USERAGENT,     CONST_STR_LEN("HTTP[\"useragent\"]"  ) },
+      { COMP_HTTP_USER_AGENT,    CONST_STR_LEN("HTTP[\"useragent\"]"  ) },
+      { COMP_HTTP_USER_AGENT,    CONST_STR_LEN("HTTP[\"user-agent\"]"  ) },
       { COMP_HTTP_COOKIE,        CONST_STR_LEN("HTTP[\"cookie\"]"     ) },
-      { COMP_HTTP_REMOTEIP,      CONST_STR_LEN("HTTP[\"remoteip\"]"   ) },
-      { COMP_HTTP_QUERYSTRING,   CONST_STR_LEN("HTTP[\"querystring\"]") },
+      { COMP_HTTP_REMOTE_IP,     CONST_STR_LEN("HTTP[\"remoteip\"]"   ) },
+      { COMP_HTTP_REMOTE_IP,     CONST_STR_LEN("HTTP[\"remote-ip\"]"   ) },
+      { COMP_HTTP_QUERY_STRING,  CONST_STR_LEN("HTTP[\"querystring\"]") },
+      { COMP_HTTP_QUERY_STRING,  CONST_STR_LEN("HTTP[\"query-string\"]") },
+      { COMP_HTTP_REQUEST_METHOD, CONST_STR_LEN("HTTP[\"request-method\"]") },
+      { COMP_HTTP_SCHEME,        CONST_STR_LEN("HTTP[\"scheme\"]"     ) },
       { COMP_UNSET, NULL, 0 },
     };
     size_t i;
@@ -1288,45 +1299,45 @@ static void yy_reduce(
   yymsp[0].minor.yy41->free(yymsp[0].minor.yy41);
   yymsp[0].minor.yy41 = NULL;
 }
-#line 1291 "configparser.c"
+#line 1302 "configparser.c"
   yy_destructor(16,&yymsp[-6].minor);
   yy_destructor(18,&yymsp[-4].minor);
   yy_destructor(19,&yymsp[-2].minor);
         break;
       case 33:
-#line 508 "./configparser.y"
+#line 520 "./configparser.y"
 {
   yygotominor.yy27 = CONFIG_COND_EQ;
 }
-#line 1301 "configparser.c"
+#line 1312 "configparser.c"
   yy_destructor(20,&yymsp[0].minor);
         break;
       case 34:
-#line 511 "./configparser.y"
+#line 523 "./configparser.y"
 {
   yygotominor.yy27 = CONFIG_COND_MATCH;
 }
-#line 1309 "configparser.c"
+#line 1320 "configparser.c"
   yy_destructor(21,&yymsp[0].minor);
         break;
       case 35:
-#line 514 "./configparser.y"
+#line 526 "./configparser.y"
 {
   yygotominor.yy27 = CONFIG_COND_NE;
 }
-#line 1317 "configparser.c"
+#line 1328 "configparser.c"
   yy_destructor(22,&yymsp[0].minor);
         break;
       case 36:
-#line 517 "./configparser.y"
+#line 529 "./configparser.y"
 {
   yygotominor.yy27 = CONFIG_COND_NOMATCH;
 }
-#line 1325 "configparser.c"
+#line 1336 "configparser.c"
   yy_destructor(23,&yymsp[0].minor);
         break;
       case 37:
-#line 521 "./configparser.y"
+#line 533 "./configparser.y"
 {
   yygotominor.yy43 = NULL;
   if (ctx->ok) {
@@ -1343,10 +1354,10 @@ static void yy_reduce(
   yymsp[0].minor.yy41->free(yymsp[0].minor.yy41);
   yymsp[0].minor.yy41 = NULL;
 }
-#line 1346 "configparser.c"
+#line 1357 "configparser.c"
         break;
       case 38:
-#line 538 "./configparser.y"
+#line 550 "./configparser.y"
 {
   if (ctx->ok) {
     if (0 != config_parse_file(ctx->srv, ctx, yymsp[0].minor.yy43->ptr)) {
@@ -1356,11 +1367,11 @@ static void yy_reduce(
     yymsp[0].minor.yy43 = NULL;
   }
 }
-#line 1359 "configparser.c"
+#line 1370 "configparser.c"
   yy_destructor(24,&yymsp[-1].minor);
         break;
       case 39:
-#line 548 "./configparser.y"
+#line 560 "./configparser.y"
 {
   if (ctx->ok) {
     if (0 != config_parse_cmd(ctx->srv, ctx, yymsp[0].minor.yy43->ptr)) {
@@ -1370,7 +1381,7 @@ static void yy_reduce(
     yymsp[0].minor.yy43 = NULL;
   }
 }
-#line 1373 "configparser.c"
+#line 1384 "configparser.c"
   yy_destructor(25,&yymsp[-1].minor);
         break;
   };
@@ -1400,11 +1411,11 @@ static void yy_parse_failed(
   while( yypParser->yyidx>=0 ) yy_pop_parser_stack(yypParser);
   /* Here code is inserted which will be executed whenever the
   ** parser fails */
-#line 107 "./configparser.y"
+#line 110 "./configparser.y"
 
   ctx->ok = 0;
 
-#line 1407 "configparser.c"
+#line 1418 "configparser.c"
   configparserARG_STORE; /* Suppress warning about unused %extra_argument variable */
 }
 
@@ -1416,6 +1427,8 @@ static void yy_syntax_error(
   int yymajor,                   /* The major type of the error token */
   YYMINORTYPE yyminor            /* The minor type of the error token */
 ){
+  UNUSED(yymajor);
+  UNUSED(yyminor);
   configparserARG_FETCH;
 #define TOKEN (yyminor.yy0)
   configparserARG_STORE; /* Suppress warning about unused %extra_argument variable */
